# encoding: utf-8

from pywinauto_recorder.player import *

def start_electron_beam():
    with UIPath(u"TESCAN Essence||Window"):
        with UIPath(u"||ToolBar->||Custom->||Group->||Custom"):
            click(u"||CheckBox#[0,0]")
